package no.rwr.awt.x;
import java.awt.*;
import java.awt.peer.*;
import java.net.*;
import java.io.FileInputStream;
import java.io.IOException;

import no.rwr.x.Display;
import no.rwr.x.Screen;
import no.rwr.x.Visual;
import no.rwr.image.BasicRasterImageConsumer;
import no.rwr.image.codec.libjpeg.JPEGImageDecoder;

public class XToolkit extends Toolkit {


    static XToolkit INSTANCE;

    /** WARNING: You MUST synchronize on this object whenever you are
        doing something that will result in Xlib calls. */
    Display display;
    XEventQueue queue;
    XGraphicsConfiguration defaultConfig;

    public XToolkit() {
	INSTANCE = this;
	display = new Display();
	synchronized (display) {
	    queue = new XEventQueue(display);
	}
    }




    /*
    protected ButtonPeer     createButton(Button         frontend);
    protected TextFieldPeer  createTextField(TextField   frontend);
    protected LabelPeer      createLabel(Label           frontend);
    protected ListPeer       createList(List             frontend);
    protected CheckboxPeer   createCheckbox(Checkbox     frontend);
    protected ScrollbarPeer  createScrollbar(Scrollbar   frontend);
    protected ScrollPanePeer createScrollPane(ScrollPane frontend);
    protected TextAreaPeer   createTextArea(TextArea     frontend);
    protected ChoicePeer     createChoice(Choice         frontend);
    */
    protected FramePeer createFrame(Frame frontend) {
	return new XFramePeer(frontend);
    }

    protected CanvasPeer createCanvas(Canvas frontend) {
	return new XCanvasPeer(frontend);
    }

    protected PanelPeer createPanel(Panel frontend) {
	return new XPanelPeer(frontend);
    }

    /*
    protected WindowPeer     createWindow(Window         frontend);
    protected DialogPeer     createDialog(Dialog         frontend);
    protected MenuBarPeer    createMenuBar(MenuBar       frontend);
    protected MenuPeer       createMenu(Menu             frontend);
    protected PopupMenuPeer  createPopupMenu(PopupMenu   frontend);
    protected MenuItemPeer   createMenuItem(MenuItem     frontend);
    protected FileDialogPeer createFileDialog(FileDialog frontend);
    protected CheckboxMenuItemPeer
	createCheckboxMenuItem(CheckboxMenuItem                   frontend);
    */


    //protected FontPeer getFontPeer(String name, int style);

    public Dimension getScreenSize() { 
	throw new UnsupportedOperationException("not implemented yet");
    }
    public int getScreenResolution() {
	throw new UnsupportedOperationException("not implemented yet");
    }
		
    public ColorModel getColorModel() {
	throw new UnsupportedOperationException("not implemented yet");
    }

    public String[] getFontList() {
	throw new UnsupportedOperationException("not implemented yet");
    }
    /*
    public FontMetrics getFontMetrics(Font font);
    */
    public void sync() {
	throw new UnsupportedOperationException("not implemented yet");
    }
    
    public Image getImage(String filename) {
	return createImage(filename);
    }
    public Image getImage(URL url) {
	throw new UnsupportedOperationException("not implemented yet");
    }

    public Image createImage(String filename) {
	BufferedImage jpeg;
	try {
	    BasicRasterImageConsumer consumer =
		new BasicRasterImageConsumer();
	    JPEGImageDecoder jid =
		new JPEGImageDecoder(new FileInputStream(filename));
	    jid.startProduction(consumer);
	    jpeg = consumer.getImage();
	} catch (IOException ioe) {
	    return null;
	}
	int w = jpeg.getWidth();
	int h = jpeg.getHeight();
	BufferedImage img =
	    getDefaultXGraphicsConfiguration().createCompatibleImage(w, h);


		
	RasterOp renderer =
	    Renderers.getInstance().createRenderer(jpeg.getColorModel(),
						   jpeg.getSampleModel(),
						   img.getColorModel(),
						   img.getSampleModel());
	if (renderer == null) {
	    throw new UnsupportedOperationException("couldn't find renderer");
	}
	renderer.filter(jpeg.getRaster(), img.getRaster());

	return img;
	

    }



    public Image createImage(URL url) {
	throw new UnsupportedOperationException("not implemented yet");
    }
    public boolean prepareImage(Image image,
				int width,
				int height,
				ImageObserver observer) {
	throw new UnsupportedOperationException("not implemented yet");
    }

    public int checkImage(Image image,
			  int width,
			  int height,
			  ImageObserver observer) {
	throw new UnsupportedOperationException("not implemented yet");
    }

    public Image createImage(ImageProducer producer) {
	throw new UnsupportedOperationException("not implemented yet");
    }

    public Image createImage(byte[] imagedata,
			     int imageoffset,
			     int imagelength) {
	throw new UnsupportedOperationException("not implemented yet");
    }

    /*
    public PrintJob getPrintJob(Frame frame,
					 String jobtitle,
					 Properties props);
    */

    public void beep() {
	throw new UnsupportedOperationException("not implemented yet");
    }

    /*
    public Clipboard getSystemClipboard();
    */


    protected EventQueue getSystemEventQueueImpl() {
	return queue;
    }
    

    XGraphicsConfiguration getDefaultXGraphicsConfiguration() {
	if (defaultConfig == null) {
	     Screen screen = display.getDefaultScreen();
	     Visual visual = screen.getRootVisual();
	     defaultConfig = new XGraphicsConfiguration(visual);

	     // ASSERT:
	     if (!defaultConfig.getVisual().getScreen().equals(screen)) {
		 String msg = "screen of graphics configuration is not " +
		     "default screen";
		 throw new Error(msg);
	     }
	}

	return defaultConfig;
    }


    /*
    public DragSourceContextPeer
	createDragSourceContextPeer(DragGestureEvent dge)
	throws InvalidDnDOperationException;

    public DragGestureRecognizer
	createDragGestureRecognizer(Class abstractRecognizerClass,
				    DragSource ds, Component c,
				    int srcActions, DragGestureListener dgl) {
	throw new UnsupportedOperationException("not implemented");
    }
    */

    
    /*
    public Map mapInputMethodHighlight(InputMethodHighlight highlight);
    */
}
