package no.rwr.image.codec.libjpeg;

import no.rwr.image.codec.*;
import no.rwr.image.*;
import java.io.*;
import java.awt.image.*;
import java.awt.color.*;



import gnu.gcj.RawData;






/* Future highlights:

   - Allow rendering directly into preexisting Raster.
   
   - The jpeg lib allows faster decompression of images at scale
   1:2, 1:4 and 1:8. Allow the Image derived frontend to this image
   producer to take advantage of this through the getScaledInstance()
   method.

   - Interruptable.

   - Detecting extended RasterImageConsumer objects for a more efficient
   API. (Delivering Raster objects rather than pixels.)

   - Selecting colorspace. (eg. grayscale is faster)


  */



/** Note: this decompresser can only serve one image consumer. Use a
    multicast adapter if multiple consumers is needed. */

public class JPEGImageDecoder implements RasterImageProducer {

    InputStream input;
    RasterImageConsumer consumer;

    RawData structs;
    private native void init();
    protected native void finalize();
    //protected native void setByteCount(int count);
    protected native void decode();



    JPEGImageDecoder(InputStream input) {
	this.input = input;
	init();
    }

    public void addConsumer(ImageConsumer consumer) {
	throw new UnsupportedOperationException("not implemented");
    }

    public void startProduction(ImageConsumer consumer) {
	throw new UnsupportedOperationException("not implemented");
    }

    public void requestTopDownLeftRightResend(ImageConsumer consumer) {
	throw new UnsupportedOperationException("not implemented");
    }

    public boolean isConsumer(ImageConsumer consumer) {
	return (this.consumer == consumer);
    }

    public void removeConsumer(ImageConsumer consumer) {
	if (isConsumer(consumer)) this.consumer = null;
    }

    public void startProduction(RasterImageConsumer consumer) {
	this.consumer = consumer;
	decode();
    }



    
    void divulgeHeaderInfo() {
	if (colorSpaceID == 0) {
	    switch (numComponents) {
	    case 3:
		colorSpaceID = ColorSpace.CS_sRGB;
		break;
	    case 1:
		colorSpaceID = ColorSpace.CS_GRAY;
		break;
	    default:
		{
		    String msg = "unsupported/unknown colorspace";
		    throw new UnsupportedOperationException(msg);
		}
	    }
	}
	ColorSpace colorSpace = ColorSpace.getInstance(colorSpaceID);
	

	// Assert number of components
	if (colorSpace.getNumComponents() != numComponents) {
	    String msg =
		"numbers of component in colorspace and from libjpeg " +
		"does not match (" + colorSpace.getNumComponents() + "!=" +
		numComponents;
	    throw new RuntimeException(msg);
	}
	
    
	final int BITS_IN_BYTE = 8; // stating the obvious
	int[] bits = new int[numComponents];
	java.util.Arrays.fill(bits, BITS_IN_BYTE);
	
	ColorModel colorModel =
	    new ComponentColorModel(colorSpace,
				    bits,
				    false, // no alpha
				    false, // not alpha premultiplied,
				    Transparency.OPAQUE,
				    DataBuffer.TYPE_BYTE // transferType
				    );
	/* Luckily, we don't have to subclass color model for it to
	   return the correct writable raster type from
	   createCompatibleWritableRaster(...) */


	consumer.setDimensions(width, height);
	consumer.setColorModel(colorModel);
	// consumer.setSampleModel(...);

	consumer.setHints(ImageConsumer.TOPDOWNLEFTRIGHT |
			  ImageConsumer.COMPLETESCANLINES |
			  ImageConsumer.SINGLEPASS |
			  ImageConsumer.SINGLEFRAME);
    }
	

    int width;
    int height;
    int numComponents;
    int scanlineStride;
    int colorSpaceID = 0;
    byte[] inputBuffer;
    
    void initInputBuffer() {
	inputBuffer = new byte[4096];
    }

    int fillBuffer() {
	try {
	    int byteCount = input.read(inputBuffer);
	    return byteCount;
	} catch (IOException ioe) {
	    return -1;
	}
    }
    
    WritableRaster rasterBuffer;
    boolean completeRasterBuffer;

    static final int MAX_SEGMENT_HEIGHT = 4;

    void initRasterBuffer() {
	if (rasterBuffer != null) {
	    completeRasterBuffer = true;
	    return;
	}
	completeRasterBuffer = false;

	// libjpeg returns RGBRGBRGB... byte data.
	rasterBuffer =
	    Raster.createInterleavedRaster(DataBuffer.TYPE_BYTE,
					   width, MAX_SEGMENT_HEIGHT,
					   numComponents, new Point());
	getScanlineStride();

    }

    WritableRaster getRaster(int y, int segmentHeight) {
	int bufferY = completeRasterBuffer ?
	    y : // pick the correct segment from the raster buffer
	    0;  // translate the raster buffer into the correct y position
	return rasterBuffer.createWritableChild(0, bufferY,
						width, segmentHeight,
						0, y,
						null // keep same band list
						);
    }


    /* FIXME: This is a general/reusable method. Move it out to a
       common spot. This implementation is based on the code in
       ComponentDataBlitOp. */
    final static int getDataOffset(Raster raster) {
	DataBuffer db = raster.getDataBuffer();
	ComponentSampleModel csm =
	    (ComponentSampleModel) raster.getSampleModel();
	int scanlineStride = csm.getScanlineStride();
	int x = raster.getMinX() - raster.getSampleModelTranslateX();
	int y = raster.getMinY() - raster.getSampleModelTranslateY();
	int numBands = csm.getNumBands();
	int offset =
	    numBands*x + scanlineStride*y +	// from sample model
	    db.getOffset();			// from data buffer
	return offset;
    }
    
    final static byte[] getDataBytes(Raster raster) {
	DataBufferByte dbb = (DataBufferByte) raster.getDataBuffer();
	return dbb.getData();
    }

    final int getScanlineStride() {
	ComponentSampleModel csm =
	    (ComponentSampleModel) rasterBuffer.getSampleModel();
	scanlineStride = csm.getScanlineStride();
	return scanlineStride;
    }
    

    void postPixels(Raster segment, int y, int h) {
	
	if (h == 0) return;
	if (h != segment.getHeight()) {
	    segment = segment.createChild(0, y, width, h,
					  0, y,
					  null // same bands
					  );
	}

	consumer.setPixels(segment);
    }

    void imageComplete() {
	consumer.imageComplete(ImageConsumer.STATICIMAGEDONE);
    }
}
