


#include <X11/Xlib.h>

#include <gcj/cni.h>
#include <java/lang/RuntimeException.h>

#include "no/rwr/x/Display.h"
#include "no/rwr/x/Screen.h"
#include "no/rwr/x/Colormap.h"
#include "no/rwr/x/XColor.h"
#include <gnu/gcj/RawData.h>

#include <vector>

#include <iostream>

jlong no::rwr::x::Colormap::allocateColorPixel(XColor* color) {
    ::Display* dpy = (::Display*) (screen->getDisplay()->display);
    ::XColor* col = (::XColor*) (color->structure);
    Status result = XAllocColor(dpy, xid, col);
    if (result == 0) {
	JvThrow(new java::lang::RuntimeException(JvNewStringLatin1("Unable to allocate color pixel.")));
    }
    return col->pixel;
}

typedef JArray<no::rwr::x::XColor*>* xcolorarray;


xcolorarray no::rwr::x::Colormap::getSharedColors() {
    ::Display* dpy = (::Display*) (screen->getDisplay()->display);
    int nCells = CellsOfScreen(ScreenOfDisplay(dpy, screen->screenNumber));

    const int NOT_SHARED = 0;
    const int SHARED = 1;

    typedef ::XColor xcolor;
    vector<xcolor> colors(nCells);
    for (int i=0; i<nCells; i++) colors[i].pixel = i;

    XQueryColors(dpy, xid, colors.begin(), nCells);


    int nShared = 0;
    for (int i=0; i<nCells; i++) {
	::XColor color = colors[i];

	if (!XAllocColor(dpy, xid, &color)) continue;

	/* FIXME: In some cases this algorithm may identify a free
           color cell as a shared one. */
	if (color.pixel != i) {
	    // Oops, the color wasn't shared. Free it.
	    XFreeColors(dpy, xid, &(color.pixel), 1, 0);
	    colors[i].flags = NOT_SHARED;
	    continue;
	}

	// FIXME: Shared or free?

	nShared++;
	colors[i].flags = SHARED;
    }

    JArray<XColor*>* shared = newXColorArray(nShared);
    int si=0;
    for (int i=0; i<nCells; i++) {
	if (colors[i].flags != SHARED) continue;
	XColor* col = elements(shared)[si++];
	gnu::gcj::RawData* colorData = col->structure;
	::XColor* colStruct = reinterpret_cast<xcolor*>(colorData);
	*colStruct = colors[i];
    }

    return shared;
}



xcolorarray no::rwr::x::Colormap::getXColors() {
    ::Display* dpy = (::Display*) (screen->getDisplay()->display);
    int nCells = CellsOfScreen(ScreenOfDisplay(dpy, screen->screenNumber));

    const int NOT_SHARED = 0;
    const int SHARED = 1;

    typedef ::XColor xcolor;
    vector<xcolor> colors(nCells);

    JArray<XColor*>* colArray = newXColorArray(nCells);

    for (int i=0; i<nCells; i++) colors[i].pixel = i;

    XQueryColors(dpy, xid, colors.begin(), nCells);

    /* TODO: The current problem with this code is that it relies on
       (color.pixel == i) as an indicator that the color is
       shared. However, (color.pixel == i), may also occur simply
       because color cell i simply was the next free in the list of
       unallocated color cells. IDEA: run through the list both
       backwards and forwards, and only pick out the colorcells that
       have been identified as shared during both passes. Reversing
       the traversal direction might prevent i from corresponding to
       the next free colorcell, atleast in one of the passes. */
    for (int i=0; i<nCells; i++) {
	::XColor color = colors[i];

	char flag = FLAG_NOT_SHARED;
	if (XAllocColor(dpy, xid, &color)) {
	    if (color.pixel == i) {
		flag = FLAG_SHARED;
	    } else {
		// Oops, the color wasn't shared. Free it.
		XFreeColors(dpy, xid, &(color.pixel), 1, 0);
	    }
	}

	// Copy color data into object in array
	XColor* col = elements(colArray)[i];
	gnu::gcj::RawData* colorData = col->structure;
	::XColor* colStruct = reinterpret_cast<xcolor*>(colorData);
	*colStruct = colors[i];
	colStruct->flags = flag;
    }

    return colArray;
}

